import { __decorate, __metadata } from 'tslib';
import { Injectable, NgModule } from '@angular/core';
import { ApolloLink, Observable } from 'apollo-link';
import { pipe, asyncScheduler, BehaviorSubject, Subject } from 'rxjs';
import { share, observeOn, distinctUntilChanged } from 'rxjs/operators';

function event() {
    return pipe(share(), observeOn(asyncScheduler));
}
let ApolloNetworkStatus = class ApolloNetworkStatus extends ApolloLink {
    constructor() {
        super();
        this.count = 0;
        this.pending$ = new BehaviorSubject(false);
        this.request$ = new Subject();
        this.success$ = new Subject();
        this.error$ = new Subject();
        this.cancel$ = new Subject();
        this.isPending = this.pending$.asObservable().pipe(event(), distinctUntilChanged());
        this.onRequest = this.request$.asObservable().pipe(event());
        this.onSuccess = this.success$.asObservable().pipe(event());
        this.onError = this.error$.asObservable().pipe(event());
        this.onCancel = this.cancel$.asObservable().pipe(event());
    }
    request(operation, forward) {
        this._onRequest({ operation });
        const subscriber = forward(operation);
        return new Observable(observer => {
            let isPending = true;
            const subscription = subscriber.subscribe({
                next: result => {
                    isPending = false;
                    this._onSuccess({ operation, result });
                    observer.next(result);
                },
                error: error => {
                    isPending = false;
                    this._onError({ operation, error });
                    observer.error(error);
                },
                complete: () => {
                    observer.complete();
                },
            });
            return () => {
                if (isPending)
                    this._onCancel({ operation });
                if (subscription)
                    subscription.unsubscribe();
            };
        });
    }
    _onRequest(data) {
        this.request$.next(data);
        this.increase();
    }
    _onSuccess(data) {
        this.success$.next(data);
        this.decrease();
    }
    _onError(data) {
        this.error$.next(data);
        this.decrease();
    }
    _onCancel(data) {
        this.cancel$.next(data);
        this.decrease();
    }
    increase() {
        this.count++;
        this.update();
    }
    decrease() {
        this.count--;
        this.update();
    }
    update() {
        this.pending$.next(this.count > 0);
    }
};
ApolloNetworkStatus = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [])
], ApolloNetworkStatus);
let ApolloNetworkStatusModule = class ApolloNetworkStatusModule {
};
ApolloNetworkStatusModule = __decorate([
    NgModule({
        providers: [ApolloNetworkStatus],
    })
], ApolloNetworkStatusModule);

/**
 * Generated bundle index. Do not edit.
 */

export { ApolloNetworkStatus, ApolloNetworkStatusModule, event };
//# sourceMappingURL=ng.apollo-network-status.js.map
