import { __extends, __decorate, __metadata } from 'tslib';
import { Injectable, NgModule } from '@angular/core';
import { Observable, ApolloLink } from 'apollo-link';
import { pipe, asyncScheduler, BehaviorSubject, Subject } from 'rxjs';
import { share, observeOn, distinctUntilChanged } from 'rxjs/operators';

function event() {
    return pipe(share(), observeOn(asyncScheduler));
}
var ApolloNetworkStatus = /** @class */ (function (_super) {
    __extends(ApolloNetworkStatus, _super);
    function ApolloNetworkStatus() {
        var _this = _super.call(this) || this;
        _this.count = 0;
        _this.pending$ = new BehaviorSubject(false);
        _this.request$ = new Subject();
        _this.success$ = new Subject();
        _this.error$ = new Subject();
        _this.cancel$ = new Subject();
        _this.isPending = _this.pending$.asObservable().pipe(event(), distinctUntilChanged());
        _this.onRequest = _this.request$.asObservable().pipe(event());
        _this.onSuccess = _this.success$.asObservable().pipe(event());
        _this.onError = _this.error$.asObservable().pipe(event());
        _this.onCancel = _this.cancel$.asObservable().pipe(event());
        return _this;
    }
    ApolloNetworkStatus.prototype.request = function (operation, forward) {
        var _this = this;
        this._onRequest({ operation: operation });
        var subscriber = forward(operation);
        return new Observable(function (observer) {
            var isPending = true;
            var subscription = subscriber.subscribe({
                next: function (result) {
                    isPending = false;
                    _this._onSuccess({ operation: operation, result: result });
                    observer.next(result);
                },
                error: function (error) {
                    isPending = false;
                    _this._onError({ operation: operation, error: error });
                    observer.error(error);
                },
                complete: function () {
                    observer.complete();
                },
            });
            return function () {
                if (isPending)
                    _this._onCancel({ operation: operation });
                if (subscription)
                    subscription.unsubscribe();
            };
        });
    };
    ApolloNetworkStatus.prototype._onRequest = function (data) {
        this.request$.next(data);
        this.increase();
    };
    ApolloNetworkStatus.prototype._onSuccess = function (data) {
        this.success$.next(data);
        this.decrease();
    };
    ApolloNetworkStatus.prototype._onError = function (data) {
        this.error$.next(data);
        this.decrease();
    };
    ApolloNetworkStatus.prototype._onCancel = function (data) {
        this.cancel$.next(data);
        this.decrease();
    };
    ApolloNetworkStatus.prototype.increase = function () {
        this.count++;
        this.update();
    };
    ApolloNetworkStatus.prototype.decrease = function () {
        this.count--;
        this.update();
    };
    ApolloNetworkStatus.prototype.update = function () {
        this.pending$.next(this.count > 0);
    };
    ApolloNetworkStatus = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ApolloNetworkStatus);
    return ApolloNetworkStatus;
}(ApolloLink));
var ApolloNetworkStatusModule = /** @class */ (function () {
    function ApolloNetworkStatusModule() {
    }
    ApolloNetworkStatusModule = __decorate([
        NgModule({
            providers: [ApolloNetworkStatus],
        })
    ], ApolloNetworkStatusModule);
    return ApolloNetworkStatusModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ApolloNetworkStatus, ApolloNetworkStatusModule, event };
//# sourceMappingURL=ng.apollo-network-status.js.map
